/***************************************************************************************
Bluetooth HID/SPPプロファイル搭載Keypad子機 for Arduino

                                                Copyright (c) 2013-2016 Wataru KUNINO
                                                http://www.geocities.jp/bokunimowakaru/
***************************************************************************************/
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

/* DF ROBOT 製液晶を使用する場合は上記を下記に入れ替えてください。
    #include <LiquidCrystalDFR.h>
    LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
*/

#define RX_MAX  17                              // 受信データ最大値
char rx_data[RX_MAX];                           // 受信データの格納用の文字列変数
int button=0;                                   // ボタン入力値
int cursor=0;                                   // 現在のカーソル位置(表示済み文字数)

int bt_mode_hid(){
    lcd_cls(1);
    lcd.print("HID Mode "); 
    if( !bt_cmd_mode('$') ){                    // ローカルコマンドモードへの移行を実行
        lcd.print("FAILED");
        return(0);
    }
    bt_cmd("SF,1");                             // 工場出荷時の設定に戻す
    bt_cmd("SM,0");                             // BluetoothのSlaveデバイスに設定する
    bt_cmd("S~,6");                             // HIDプロファイルを選択する
    bt_cmd("R,1");                              // 再起動
    lcd.print("DONE");
    delay(1000);                                // 再起動待ち
    return(1);
}

int bt_mode_spp(){
    lcd_cls(1);
    lcd.print("SPP Mode "); 
    if( !bt_cmd_mode('$') ){                    // ローカルコマンドモードへの移行を実行
        lcd.print("FAILED");
        return(0);
    }
    bt_cmd("SF,1");                             // 工場出荷時の設定に戻す
    bt_cmd("SM,0");                             // BluetoothのSlaveデバイスに設定する
    bt_cmd("S~,0");                             // HIDプロファイルを選択する
    bt_cmd("R,1");                              // 再起動
    lcd.print("DONE");
    delay(1000);                                // 再起動待ち
    return(1);
}

void setup(){
    lcd.begin(16, 2);
    lcd.print("Bluetooth LCD KB");              // タイトル文字を表示
    Serial.begin(115200);                       // シリアルポートの初期化
    lcd.setCursor(0,1);
    lcd.print("<- LCD    Key ->");
    while( button == 0){
        button = lcd.readButtons();
        switch( button ){                       // 押されたボタンに対して
            case BUTTON_LEFT:
                bt_mode_spp();                  // SPPモードに設定
                break;
            case BUTTON_RIGHT:
                bt_mode_hid();                  // HIDモードに設定
                break;
            default:
                button=0;
                break;
        }
    }
}

void loop(){
    if( Serial.available() ){                   // シリアル受信があった時
        char c = Serial.read();                 // cに受信値を代入
        if( cursor==0 ){
            if( c == 0x1B ){                    // ESCコードだったとき
                int temp = (int)(getTemp()+0.5);// 温度を取得
                lcd_cls(1);                     // 液晶の2行目の文字を消去
                lcd.print("Room Temp : ");      // 液晶に「Room Temp : 」を表示
                lcd.print(temp);                // 温度を表示
                Serial.println(temp);           // 温度を送信
            }
            lcd_cls(0);                         // 表示済文字数0の時に文字を消去する
        }
        if( cursor < 16){                       // 表示済みの文字数が15文字以内の時
            lcd.setCursor(cursor,0);            // 液晶の表示位置にカーソルを移動
            if( isprint(c) ) lcd.print(c);      // 表示可能文字の場合は文字を表示
            else lcd.print(' ');                // 表示不可能な場合は空白を表示
            cursor++;                           // 表示済み文字数を1つ増やす
        }
        if( c=='\n' || c=='\r') cursor=0;       // 改行の時に表示済み文字数を0に
    }
    button=lcd.readButtons();                   // ボタン値をbuttonへ代入
    if(button){
        lcd_cls(1);
        lcd.print("Keyboard: ");
        switch( button ){                       // 押されたボタンに対して
            case BUTTON_UP:
                lcd.print("UP    ");            // 上ボタンの時にUPと表示
                Serial.write(14);               // コード14をシリアル送信
                break;
            case BUTTON_DOWN:
                lcd.print("DOWN  ");            // 下ボタンの時にDOWNと表示
                Serial.write(12);               // コード12をシリアル送信
                break;
            case BUTTON_LEFT:
                lcd.print("LEFT  ");            // 左ボタンの時にLEFTと表示
                Serial.write(11);               // コード11をシリアル送信
                break;
            case BUTTON_RIGHT:
                lcd.print("RIGHT ");            // 右ボタンの時にRIGHTと表示
                Serial.write(7);                // コード7をシリアル送信
                break;
            case BUTTON_SELECT:
                lcd.print("SELECT");            // SELECTボタンの時にSELECTと表示
                Serial.write(13);               // コード13をシリアル送信
                break;
        }
        while(lcd.readButtons()) delay(100);    // ボタンを放すまで待機
    }
}
